'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactAddonsPureRenderMixin = require('react-addons-pure-render-mixin');

var _reactAddonsPureRenderMixin2 = _interopRequireDefault(_reactAddonsPureRenderMixin);

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var css = {
  valid: 'valid',
  invalid: 'invalid',
  dirty: 'dirty',
  pristine: 'pristine'
};

function makeErrorsList(errors) {
  if (errors.length > 0) {
    return _react2.default.createElement(
      'ul',
      { className: 'errors' },
      errors.map(function (e, i) {
        return _react2.default.createElement(
          'li',
          { key: i },
          e
        );
      })
    );
  } else {
    return null;
  }
}

function showErrors(props) {
  var errOnDirty = props.errOnDirty;
  var dirty = props.dirty;

  return !errOnDirty || dirty;
}

function getFieldClassName(props) {
  var dirty = props.dirty;
  var errors = props.errors;
  var moreErrors = props.moreErrors;
  var _props$className = props.className;
  var className = _props$className === undefined ? '' : _props$className;

  var result = className;
  if (dirty) {
    result = result.replace(css.pristine, '').concat(' ' + css.dirty);
  } else {
    result = result.replace(css.dirty, '').concat(' ' + css.pristine);
  }
  if (errors || moreErrors) {
    result = result.replace(css.valid, '').concat(' ' + css.invalid);
  } else {
    result = result.replace(css.invalid, '').concat(' ' + css.valid);
  }
  return result;
}

function renderWidget(widget, props, children) {
  if (props.fields) {
    children = _react2.default.Children.map(children, function (el) {
      return _react2.default.cloneElement(el, props.fields[el.props.value], el.props.children);
    });
  }
  return widget && _react2.default.createElement(widget, (0, _utils.filterProps)(widget, props), children);
}

function labeledWidget(props) {
  var _props$widget = props.widget;
  var widget = _props$widget === undefined ? 'input' : _props$widget;
  var label = props.label;
  var children = props.children;

  var className = getFieldClassName(props);
  var finalProps = _extends({}, props, { className: className });
  if (props.type === 'file') delete finalProps.value;

  if (label) {
    return _react2.default.createElement(
      'label',
      null,
      _react2.default.createElement(
        'span',
        null,
        label
      ),
      renderWidget(widget, finalProps, children)
    );
  } else {
    return renderWidget(widget, finalProps, children);
  }
}

exports.default = _react2.default.createClass({
  displayName: 'HothField',

  mixins: [_reactAddonsPureRenderMixin2.default],

  render: function render() {
    var props = this.props;
    var widget = props.widget;
    var errors = props.errors;
    var moreErrors = props.moreErrors;
    var children = props.children;

    if (widget === 'select') {
      children = _react2.default.Children.map(children, function (el) {
        return _react2.default.cloneElement(el, props.fields[el.props.value]);
      });
    }
    return _react2.default.createElement(
      'div',
      { className: 'form-group' },
      labeledWidget(props),
      showErrors(props) && makeErrorsList([].concat(errors, moreErrors).filter(function (i) {
        return i;
      }))
    );
  }
});