'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utils = require('../utils');

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.default = _react2.default.createClass({
  displayName: 'HothForm',

  updateForm: function updateForm(fields, element) {
    var _props = this.props;
    var onChange = _props.onChange;
    var validate = _props.validate;

    var changes = (0, _utils.changeValue)(fields, element);
    changes = (0, _utils.additionalChanges)(changes, onChange, element);
    changes = (0, _utils.performValidation)(changes, validate);
    this.setState(changes);
  },

  resetFields: function resetFields() {
    var _props2 = this.props;
    var onChange = _props2.onChange;
    var validate = _props2.validate;

    var changes = (0, _utils.resetFields)(this.state.fields);
    changes = (0, _utils.additionalChanges)(changes, onChange);
    changes = (0, _utils.performValidation)(changes, validate);
    this.setState(changes);
  },

  handleFocusChange: function handleFocusChange(e) {
    var changes = (0, _utils.additionalChanges)(this.state.fields, function () {
      return _defineProperty({}, e.target.name, { active: e.type === 'focus' });
    });
    this.setState({ fields: changes });
  },

  getFormState: function getFormState() {
    var _state = this.state;
    var fields = _state.fields;
    var errors = _state.errors;
    var reset = _state.reset;

    return { fields: fields, errors: errors, reset: reset };
  },

  onFieldChange: function onFieldChange(fn, e) {
    fn && fn(e);
    this.updateForm(this.state.fields, e.target);
  },

  onFieldFocus: function onFieldFocus(fn, e) {
    fn && fn(e);
    this.handleFocusChange(e);
  },

  onFieldBlur: function onFieldBlur(fn, e) {
    fn && fn(e);
    this.handleFocusChange(e);
  },

  onFormSubmit: function onFormSubmit(e) {
    e.preventDefault();
    var onSubmit = this.props.onSubmit;

    onSubmit && onSubmit(this.getFormState());
  },

  componentWillMount: function componentWillMount() {
    var fields = this.props.fields;

    if (!fields) throw new Error('You forgot to provide form fields');
    var initializedFields = (0, _utils.initFields)(this.props.fields);
    this.setState({ fields: initializedFields, reset: this.resetFields });
    this.updateForm(initializedFields);
  },

  render: function render() {
    var _props3 = this.props;
    var children = _props3.children;
    var id = _props3.id;


    return _react2.default.createElement(
      'form',
      { id: id, onSubmit: this.onFormSubmit },
      this.iterateAndInjectProps(children)
    );
  },

  iterateAndInjectProps: function iterateAndInjectProps(children) {
    var _this = this;

    return _react2.default.Children.map(children, function (el) {
      if (el && el.props) {
        if (_config2.default.fieldGroupClassName.test(el.props.className)) {
          el = _react2.default.cloneElement(el, el.props, _this.iterateAndInjectProps(el.props.children));
        }
        var getHoth = el.props['hoth'];
        var hothProps = {};
        if (getHoth) {
          hothProps = getHoth(_this.getFormState(), el);
        }
        var fieldProps = _this.state.fields[el.props.name];
        if (fieldProps) {
          fieldProps = _extends({}, fieldProps, el.props, hothProps);
          var moreErrors = _this.props.moreErrors;

          var props = {
            onChange: _this.onFieldChange.bind(null, fieldProps.onChange),
            onFocus: _this.onFieldFocus.bind(null, fieldProps.onFocus),
            onBlur: _this.onFieldBlur.bind(null, fieldProps.onBlur),
            moreErrors: moreErrors && moreErrors[el.props.name]
          };
          if (fieldProps.fields) {
            if (el.type === 'select' || el.props.widget === 'select' || fieldProps.widget === 'select') {
              var value = fieldProps.value;
              if (!el.props.mutiple && !fieldProps.multiple) {
                value = value[0];
              }
              props = _extends({}, fieldProps, props, { value: value });
              /*} else if (!el.props.value) { // I've forgotten why need this line :D
                const {value, ...newFieldProps} = fieldProps;
                props = {...newFieldProps, ...props};     */
            } else {
              props = _extends({}, fieldProps.fields[el.props.value], props);
            }
          } else {
            props = _extends({}, fieldProps, props);
          }
          return _react2.default.cloneElement(el, (0, _utils.filterProps)(el, props));
        } else {
          if (getHoth) {
            return _react2.default.cloneElement(el, (0, _utils.filterProps)(el, hothProps));
          } else {
            return el;
          }
        }
      } else {
        return el;
      }
    });
  }
});