'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _enzyme = require('enzyme');

var _chaiEnzyme = require('chai-enzyme');

var _chaiEnzyme2 = _interopRequireDefault(_chaiEnzyme);

var _Field = require('../containers/Field');

var _Field2 = _interopRequireDefault(_Field);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use((0, _chaiEnzyme2.default)());

describe('Field component', function () {
  it('renders `widget` element (if not specified - `input`)', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, null)).find('input')).to.exist;
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { widget: 'select' })).find('select')).to.exist;
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { widget: 'textarea' })).find('textarea')).to.exist;
  });

  it('renders `label` element', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { label: 'Hello, field!' })).find('label')).to.have.text('Hello, field!');
  });

  it('renders `errors` and `moreErrors`', function () {
    var errors = (0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { errors: ['No', 'one'] })).find('.errors li').map(function (i) {
      return i.text();
    });
    (0, _chai.expect)(errors).to.eql(['No', 'one']);

    var moreErrors = (0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { moreErrors: ['should', null, 'pass'] })).find('.errors li').map(function (i) {
      return i.text();
    });
    (0, _chai.expect)(moreErrors).to.eql(['should', 'pass']);

    var combined = (0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { errors: ['the', undefined], moreErrors: 'gates' })).find('.errors li').map(function (i) {
      return i.text();
    });
    (0, _chai.expect)(combined).to.eql(['the', 'gates']);

    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, null)).find('.errors')).to.not.exist;
  });

  it('doesn`t render errors if `errOnDirty` is true, field is not dirty and has errors', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { errOnDirty: true, dirty: true, errors: ['No', 'one'] })).find('.errors')).to.exist;
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { errOnDirty: true, dirty: false, errors: ['No', 'one'] })).find('.errors')).to.not.exist;
  });

  it('injects all props to the `widget` element if it is custom React component', function () {
    function MyCustom(props) {
      return _react2.default.createElement(
        'div',
        null,
        'custom'
      );
    }
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { widget: MyCustom, id: 'name', className: 'myName', value: 'john', label: 'User' })).find('MyCustom').props()).to.eql({
      id: 'name',
      value: 'john',
      className: 'myName pristine valid',
      label: 'User',
      widget: MyCustom,
      children: undefined
    });
  });

  it('injects filtered props to the `widget` element if it is plain old form element', function () {
    var injectedProps = {
      id: 'name',
      className: 'myName',
      value: 'john',
      label: 'User',
      hoth: 'test',
      extra: 'test'
    };
    var expectedProps = {
      id: 'name',
      value: 'john',
      className: 'myName pristine valid',
      children: undefined
    };
    function testFiltering(widget) {
      (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, _extends({ widget: widget }, injectedProps))).find(widget).props()).to.eql(expectedProps);
    }

    testFiltering('input');
    testFiltering('select');
    testFiltering('button');
    testFiltering('textarea');
  });

  it('appends current className with `dirty/pristine` based on `dirty` prop', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { dirty: true })).find('input')).to.have.prop('className', ' dirty valid');

    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { dirty: false })).find('input')).to.have.prop('className', ' pristine valid');
  });

  it('appends current className with `valid/invalid` based on `errors` and `moreErors` props', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { errors: true })).find('input')).to.have.prop('className', ' pristine invalid');

    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { moreErrors: true })).find('input')).to.have.prop('className', ' pristine invalid');

    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, null)).find('input')).to.have.prop('className', ' pristine valid');
  });

  it('does not set `value` prop if `type == file`', function () {
    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { type: 'file', value: 'somefile' })).find('input')).to.not.have.prop('value', 'somefile');

    (0, _chai.expect)((0, _enzyme.shallow)(_react2.default.createElement(_Field2.default, { type: 'file', value: '' })).find('input')).to.not.have.prop('value', '');
  });

  it('render children with `fields` prop', function () {
    var fields = {
      red: { id: 'red' },
      green: { id: 'green', disabled: true },
      blue: {}
    };
    var f = (0, _enzyme.shallow)(_react2.default.createElement(
      _Field2.default,
      { fields: fields },
      _react2.default.createElement('option', { value: 'red' }),
      _react2.default.createElement('option', { value: 'green' }),
      _react2.default.createElement('option', { value: 'blue' })
    ));
    (0, _chai.expect)(f.find('#red')).to.exist;
    (0, _chai.expect)(f.find('#green')).to.have.prop('disabled', true);
  });
});