'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _sinon = require('sinon');

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _enzyme = require('enzyme');

var _chaiEnzyme = require('chai-enzyme');

var _chaiEnzyme2 = _interopRequireDefault(_chaiEnzyme);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _chaiSubset = require('chai-subset');

var _chaiSubset2 = _interopRequireDefault(_chaiSubset);

var _Form = require('../containers/Form');

var _Form2 = _interopRequireDefault(_Form);

var _Field = require('../containers/Field');

var _Field2 = _interopRequireDefault(_Field);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use((0, _chaiEnzyme2.default)());
_chai2.default.use(_sinonChai2.default);
_chai2.default.use(_chaiSubset2.default);

var fields = {
  username: 'john',
  sex: { fields: ['male', 'female'], value: 'male' },
  agree: false,
  hands: { fields: ['left', 'right'], value: [] },
  fruits: { fields: ['tomato', 'potato'], value: ['tomato'] },
  pets: { fields: ['rat', 'python'], value: ['rat'], className: 'myPets' }
};

function onChange(p) {
  return {
    fruits: {
      fields: { potato: { disabled: true } }
    },
    wontBeInjected: {
      foo: 'bar'
    }
  };
}

function validate(fields) {
  return {
    username: 'Too many johnes',
    wontBeInjected: 'foo'
  };
}

function disableSubmit(fields) {
  return {
    disabled: true
  };
}

function createForm(onChange, validate, moreErrors) {
  return (0, _enzyme.shallow)(_react2.default.createElement(
    _Form2.default,
    { fields: fields, onChange: onChange, validate: validate, moreErrors: moreErrors },
    _react2.default.createElement(_Field2.default, { id: 'username', name: 'username' }),
    _react2.default.createElement('input', { id: 'male', name: 'sex', value: 'male', type: 'radio' }),
    _react2.default.createElement('input', { id: 'female', name: 'sex', value: 'female', type: 'radio' }),
    _react2.default.createElement('input', { id: 'agree', name: 'agree', type: 'checkbox' }),
    _react2.default.createElement('input', { id: 'left', name: 'hands', value: 'left', type: 'checkbox' }),
    _react2.default.createElement('input', { id: 'right', name: 'hands', value: 'right', type: 'checkbox' }),
    _react2.default.createElement(
      'fieldset',
      { className: 'extra field-group' },
      _react2.default.createElement(
        'legend',
        null,
        'Extra'
      ),
      _react2.default.createElement(
        'select',
        { id: 'fruits', name: 'fruits' },
        _react2.default.createElement(
          'option',
          { id: 'tomato', value: 'tomato' },
          'Tomato'
        ),
        _react2.default.createElement(
          'option',
          { value: 'potato' },
          'Potato'
        )
      ),
      _react2.default.createElement(
        'select',
        { id: 'pets', name: 'pets', multiple: true },
        _react2.default.createElement(
          'option',
          { value: 'rat' },
          'Rat'
        ),
        _react2.default.createElement(
          'option',
          { value: 'python' },
          'Python'
        )
      )
    ),
    _react2.default.createElement('input', { id: 'excess', name: 'excess' }),
    _react2.default.createElement(
      'button',
      { hoth: disableSubmit },
      'Submit'
    )
  ));
}

describe('Form container', function () {
  it('requires `fields` prop', function () {
    (0, _chai.expect)(function () {
      (0, _enzyme.shallow)(_react2.default.createElement(_Form2.default, null));
    }).to.throw(Error);
    (0, _chai.expect)(function () {
      (0, _enzyme.shallow)(_react2.default.createElement(_Form2.default, { fields: fields }));
    }).to.not.throw(Error);
  });

  it('handles focus changes', function () {
    var form = createForm();
    form.find('#username').simulate('focus', { target: { name: 'username' }, type: 'focus' });
    (0, _chai.expect)(form.state()).to.containSubset({
      fields: {
        username: { active: true }
      }
    });
    form.find('#username').simulate('blur', { target: { name: 'username' }, type: 'blur' });
    (0, _chai.expect)(form.state()).to.containSubset({
      fields: {
        username: { active: false }
      }
    });
  });

  it('can reset fields to initial values (and also calls `onChange` and `validate` in process)', function () {
    var form = createForm(onChange, validate);
    form.find('#username').simulate('change', { target: { name: 'username', value: 'mark' } });
    form.state().reset();
    (0, _chai.expect)(form.state()).to.containSubset({
      fields: {
        username: {
          value: 'john',
          dirty: false,
          errors: ['Too many johnes']
        }
      }
    });
    (0, _chai.expect)(form.find('button')).to.have.prop('disabled', true);
  });

  describe('when has been mounted', function () {
    it('initializes fields', function () {
      var form = createForm();

      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          username: { initialValue: 'john', value: 'john' },
          sex: { fields: { male: { checked: true }, female: { checked: false } }, initialValue: ['male'], value: ['male'] },
          agree: { initialValue: false, value: false, checked: false },
          hands: { fields: { left: { checked: false }, right: { checked: false } }, initialValue: [], value: [] },
          fruits: { fields: { tomato: { checked: true }, potato: { checked: false } }, initialValue: ['tomato'], value: ['tomato'] },
          pets: { fields: { rat: { checked: true }, python: { checked: false } }, initialValue: ['rat'], value: ['rat'], className: 'myPets' },
          nonFieldErrors: { errors: null }
        },
        errors: false
      });
    });

    it('calls `onChange` function if it is provided', function () {
      var form = createForm(onChange);
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          fruits: { fields: { potato: { disabled: true } } }
        }
      });
      (0, _chai.expect)(form.state()).to.not.containSubset({
        fields: {
          wontBeInjected: { foo: 'bar' }
        }
      });
    });

    it('calls `validate` function if it is provided', function () {
      var form = createForm(undefined, validate);
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          username: { errors: ['Too many johnes'] }
        },
        errors: true
      });
      (0, _chai.expect)(form.state()).to.not.containSubset({
        fields: {
          wontBeInjected: { errors: ['foo'] }
        }
      });
    });

    it('initializes `non-field-errors` field with the key from config object', function () {
      var savedKey = _config2.default.nonFieldErrorsKey;
      _config2.default.nonFieldErrorsKey = 'formErrors';
      var form = createForm();
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          formErrors: { errors: null }
        }
      });
      _config2.default.nonFieldErrorsKey = savedKey;
    });
  });

  describe('on render', function () {
    it('provides direct children which have prop `name` with corresponding field props', function () {
      var form = createForm(onChange);
      (0, _chai.expect)(form.find("#username")).to.have.prop('value', 'john');
      (0, _chai.expect)(form.find("#male")).to.have.prop('checked', true);
      (0, _chai.expect)(form.find("#female")).to.have.prop('checked', false);
      (0, _chai.expect)(form.find("#agree")).to.have.prop('value', false);
      (0, _chai.expect)(Object.keys(form.find("#excess").props()).length).to.eql(2);
      (0, _chai.expect)(Object.keys(form.find("button").props()).length).to.eql(3); // children 'Submit'
    });

    it('provides indirect children which parents have className equal to `fieldGroupClassName` with corresponding field props', function () {
      var form = createForm();
      (0, _chai.expect)(form.find("#fruits")).to.have.prop('value', 'tomato');
      (0, _chai.expect)(form.find("#pets")).to.have.prop('className', 'myPets');
    });

    it('provides children which have prop `hoth` with props returned by calling `hoth` function', function () {
      var spied = (0, _sinon.spy)(disableSubmit);
      var form = createForm();

      // Something strange here - it shows that spy is not being called, but injects props correctly
      // expect(spied).to.have.been.calledWith({
      //   fields: {
      //     username: {initialValue: 'john', value: 'john'},
      //     sex: {fields: {male: {checked: true}, female: {checked: false}}, initialValue: 'male', value: 'male'},
      //     agree: {checked: false, initialValue: false, value: false},
      //     hands: {fields: {left: {checked: false}, right: {checked: false}}, initialValue: [], value: []},
      //     fruits: {fields: {tomato: {checked: true}, potato: {checked: false}}, initialValue: 'tomato', value: 'tomato'},
      //     pets: {fields: {rat: {checked: true}, python: {checked: false}}, initialValue: ['rat'], value: ['rat']},
      //     nonFieldErrors: {errors: null}
      //   },
      //   errors: false
      // });
      (0, _chai.expect)(form.find('button')).to.have.prop('disabled', true);
    });
  });

  describe('onFieldChange function', function () {
    // Something strange again.. Maybe I do it wrong, but seems like it works.
    it.skip('calls `onChange` function if it is provided', function () {
      var spied = (0, _sinon.spy)(onChange);
      var form = createForm(spied);
      form.find('#username').simulate('change', { target: { name: 'username', value: 'mark' } });
      (0, _chai.expect)(spied).to.have.been.calledWith({
        username: { initialValue: 'john', value: 'mark', dirty: true },
        sex: { fields: { male: { checked: true }, female: { checked: false } }, initialValue: 'male', value: 'male' },
        agree: { checked: false, initialValue: false, value: false },
        hands: { fields: { left: { checked: false }, right: { checked: false } }, initialValue: [], value: [] },
        fruits: { fields: { tomato: { checked: true }, potato: { checked: false } }, initialValue: 'tomato', value: ['tomato'] },
        pets: { fields: { rat: { checked: true }, python: { checked: false } }, initialValue: ['rat'], value: ['rat'], className: 'myPets' },
        nonFieldErrors: { errors: null }
      }, {
        name: 'username',
        value: 'mark'
      });
    });

    it('calls `validate` function if it is provided', function () {
      var spied = (0, _sinon.spy)(validate);
      var form = createForm(null, spied);
      form.find('#username').simulate('change', { target: { name: 'username', value: 'mark' } });
      (0, _chai.expect)(spied).to.have.been.calledWith({
        username: { initialValue: 'john', value: 'mark', errors: ['Too many johnes'], dirty: true },
        sex: { fields: { male: { checked: true }, female: { checked: false } }, initialValue: ['male'], value: ['male'] },
        agree: { checked: false, initialValue: false, value: false },
        hands: { fields: { left: { checked: false }, right: { checked: false } }, initialValue: [], value: [] },
        fruits: { fields: { tomato: { checked: true }, potato: { checked: false } }, initialValue: ['tomato'], value: ['tomato'] },
        pets: { fields: { rat: { checked: true }, python: { checked: false } }, initialValue: ['rat'], value: ['rat'], className: 'myPets' },
        nonFieldErrors: { errors: null }
      });
    });

    it('injects `moreErrors` keys values into elements with corresponding `name` props', function () {
      var form = createForm(null, null, { username: 'This name was already taken' });
      (0, _chai.expect)(form.find('#username')).to.have.prop('moreErrors', 'This name was already taken');
    });

    it('handles input changes', function () {
      var form = createForm();
      form.find('#username').simulate('change', { target: {
          name: 'username',
          value: 'mark'
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          username: { value: 'mark' }
        }
      });
    });

    it('handles radio changes', function () {
      var form = createForm();
      form.find('#female').simulate('change', { target: {
          name: 'sex',
          value: 'female',
          checked: true
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          sex: { value: 'female' }
        }
      });
    });

    it('handles single checkbox changes', function () {
      var form = createForm();
      form.find('#agree').simulate('change', { target: {
          name: 'agree',
          type: 'checkbox',
          value: 'false',
          checked: true
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          agree: { value: true }
        }
      });
    });

    it('handles multiple checkbox changes', function () {
      var form = createForm();
      form.find('#left').simulate('change', { target: {
          name: 'hands',
          type: 'checkbox',
          value: 'left',
          checked: true
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          hands: {
            fields: { left: { checked: true } }
          }
        }
      });
    });

    it('handles select changes', function () {
      var form = createForm();
      form.find('#fruits').simulate('change', { target: {
          name: 'fruits',
          type: 'select',
          options: [{ selected: false, value: 'tomato' }, { selected: true, value: 'potato' }]
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          fruits: {
            value: ['potato']
          }
        }
      });
    });

    it('handles multiple select changes', function () {
      var form = createForm();
      form.find('#pets').simulate('change', { target: {
          name: 'pets',
          type: 'select-multiple',
          options: [{ selected: true, value: 'rat' }, { selected: true, value: 'python' }]
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          pets: {
            value: ['rat', 'python']
          }
        }
      });
    });

    it('handles file field changes', function () {
      var fields = {
        photo: { id: 'photo', type: 'file' }
      };
      var form = (0, _enzyme.shallow)(_react2.default.createElement(
        _Form2.default,
        { fields: fields },
        _react2.default.createElement(_Field2.default, { name: 'photo' })
      ));
      form.find('#photo').simulate('change', { target: {
          name: 'photo',
          type: 'file',
          files: ['myFile1']
        } });
      (0, _chai.expect)(form.state()).to.containSubset({
        fields: {
          photo: {
            value: ['myFile1']
          }
        }
      });
    });
  });
});