'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.injectedProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.initFields = initFields;
exports.changeValue = changeValue;
exports.resetFields = resetFields;
exports.additionalChanges = additionalChanges;
exports.performValidation = performValidation;
exports.isPlainOldType = isPlainOldType;
exports.filterProps = filterProps;

var _mergeWith = require('lodash/mergeWith');

var _mergeWith2 = _interopRequireDefault(_mergeWith);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isEqual = require('lodash/isEqual');

var _isEqual2 = _interopRequireDefault(_isEqual);

var _sortBy = require('lodash/sortBy');

var _sortBy2 = _interopRequireDefault(_sortBy);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function initFields(fields) {
  var result = {};
  Object.keys(fields).forEach(function (key) {
    var schema = fields[key];
    var field = {};

    if ((0, _isPlainObject2.default)(schema)) {
      field = _extends({}, schema);
    } else {
      field.value = schema;
      if (typeof schema === 'boolean') {
        field.checked = schema;
      }
    }

    if (!field.hasOwnProperty('initialValue')) {
      field.initialValue = field.value === undefined ? '' : field.value;
    }
    if (!field.hasOwnProperty('value')) {
      field.value = field.initialValue;
    }

    if (field.fields) {
      if (typeof field.initialValue === 'string') field.initialValue = [field.initialValue];
      if (typeof field.value === 'string') field.value = [field.value];

      if (!(0, _isPlainObject2.default)(field.fields)) {
        (function () {
          var obj = {};
          field.fields.forEach(function (f) {
            return obj[f] = {};
          });
          field.fields = obj;
        })();
      }

      Object.keys(field.fields).forEach(function (f) {
        field.fields[f] = _extends({}, field.fields[f], {
          checked: isChecked(f, field.value)
        });
      });
    } else {
      if (typeof field.value === 'boolean') {
        field.checked = field.value;
      }
    }
    result[key] = field;
  });
  result[_config2.default.nonFieldErrorsKey] = { errors: null };
  return result;
}

function changeValue(fields, element) {
  var result = _extends({}, fields);
  if (element) {
    var newFields = elementToField(element);
    result = iterateThroughFields(result, newFields, setValue);
  }
  return result;
}

function resetFields(fields) {
  var newFields = {};
  Object.keys(fields).forEach(function (k) {
    return newFields[k] = { value: fields[k].initialValue };
  });
  return iterateThroughFields(fields, newFields, setValue);
}

function additionalChanges(fields, onChange, element) {
  var newFields = onChange && onChange(fields, element) || {};
  return iterateThroughFields(fields, newFields, setValue);
}

function performValidation(fields, validate) {
  var newFields = validate && validate(fields) || {};
  var adjustedFields = {};
  Object.keys(fields).forEach(function (k) {
    adjustErrorField(adjustedFields, newFields, fields[k], k);
  });
  newFields = iterateThroughFields(fields, adjustedFields);
  var errors = Object.keys(newFields).some(function (k) {
    return newFields[k].errors;
  });
  return {
    errors: errors,
    fields: newFields
  };
}

var injectedProps = exports.injectedProps = ['initialValue', 'active', 'dirty', 'errors', 'moreErrors', 'fields', 'validation', 'widget', 'label', 'hoth', 'extra'];

function isPlainOldType(el) {
  if (!el) return false;
  var type = typeof el === 'string' ? el : el.type;
  switch (type) {
    case 'input':
    case 'button':
    case 'select':
    case 'textarea':
      return true;

    default:
      return false;
  }
}

function filterProps(el, props) {
  var blockedProps = arguments.length <= 2 || arguments[2] === undefined ? injectedProps : arguments[2];

  var result = _extends({}, props);
  if (isPlainOldType(el)) {
    blockedProps.forEach(function (p) {
      return delete result[p];
    });
  }
  return result;
}

function adjustErrorField(adjustedFields, validationFields, oldFields, key) {
  var errors = null;
  var value = validationFields[key];
  if (value) {
    value = value.toJS && value.toJS() || value;
    if (typeof value === 'string') {
      errors = [value];
    } else {
      errors = value;
    }
  }
  if (value || oldFields.errors) {
    adjustedFields[key] = { errors: errors };
  }
}

function iterateThroughFields(oldFields, newFields, fn) {
  var result = _extends({}, oldFields);
  Object.keys(newFields).forEach(function (key) {
    var oldProps = oldFields[key];
    if (oldProps) {
      var newProps = newFields[key];
      var newField = (0, _mergeWith2.default)({}, oldProps, newProps, replaceArrays);
      if (fn) {
        newField = fn(newField, newProps);
      }
      result[key] = newField;
    }
  });
  return result;
}

function setValue(newField, newProps) {
  // If value is provided with props, then update all `checked` props if they are exist.
  // If value is NOT provided, but `checked` IS, then update `value` prop.
  if (newProps.hasOwnProperty('value')) {
    if (newField.fields) {
      Object.keys(newField.fields).forEach(function (k) {
        newField.fields[k].checked = newField.value.indexOf(k) !== -1;
      });
    } else {
      if (newField.hasOwnProperty('checked')) {
        newField.checked = newField.value;
      }
    }
  } else {
    if (newField.fields) {
      newField.value = Object.keys(newField.fields).filter(function (i) {
        return newField.fields[i].checked;
      });
    } else {
      if (newProps.hasOwnProperty('checked')) {
        newField.value = newProps.checked;
      }
    }
  }
  newField.dirty = isDirty(newField.value, newField.initialValue);
  return newField;
}

function isChecked(fieldName, value) {
  if (!value) return false;
  return value.indexOf(fieldName) !== -1;
}

function isDirty(val, initVal) {
  if (typeof val === 'boolean') {
    return val !== initVal;
  } else {
    return !(0, _isEqual2.default)((0, _sortBy2.default)(val), (0, _sortBy2.default)(initVal));
  }
}

function replaceArrays(objValue, srcValue) {
  if ((0, _isArray2.default)(objValue)) {
    return srcValue;
  }
}

function elementToField(element) {
  var name = element.name;
  var type = element.type;
  var value = element.value;

  var field = {};
  switch (type) {
    case 'checkbox':
      if (value === 'true' || value === 'false') {
        field.value = element.checked;
      } else {
        field.fields = _defineProperty({}, value, { checked: element.checked });
      }
      break;

    case 'radio':
      field.value = [value];
      break;

    case 'select':
    case 'select-multiple':
      field.value = optionsToValue(element.options);
      break;

    case 'file':
      field.value = element.files;
      break;

    case undefined:
    default:
      field.value = value;
  }
  return _defineProperty({}, name, field);
}

function optionsToValue(options) {
  return [].concat(_toConsumableArray(options)).filter(function (o) {
    return o.selected;
  }).map(function (o) {
    return o.value.toString();
  });
}